/*
 * unity-webapps-context-daemon.c
 * Copyright (C) Canonical LTD 2011
 *
 * Author: Robert Carr <racarr@canonical.com>
 * 
 unity-webapps is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * unity-webapps is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.";
 */

#include "unity-webapps-service.h"
#include "gen-webapps-service.xml.h"

static GMainLoop *mainloop = NULL;

static GDBusNodeInfo *introspection_data = NULL;

static void
handle_method_call (GDBusConnection *connection,
		    const gchar *sender,
		    const gchar *object_path,
		    const gchar *interface_name,
		    const gchar *method_name,
		    GVariant *parameters,
		    GDBusMethodInvocation *invocation,
		    gpointer user_data)
{
  
  if (g_strcmp0 (method_name, "HelloWorld") == 0)
    {
      g_dbus_method_invocation_return_value (invocation, g_variant_new("(s)", "Hello World"));
    }
}

static const GDBusInterfaceVTable interface_vtable =
  {
    handle_method_call,
    NULL,
    NULL
  };


static void
on_bus_acquired (GDBusConnection *connection,
		 const gchar *name,
		 gpointer user_data)
{
  guint reg_id;

  reg_id = g_dbus_connection_register_object (connection, 
					      "/com/canonical/Unity/Webapps/Service",
					      introspection_data->interfaces[0],
					      &interface_vtable,
					      NULL /* user data */,
					      NULL /* user data free */,
					      NULL /* Error */);
  
  g_assert (reg_id > 0);
}

static void
on_name_acquired (GDBusConnection *connection,
		  const gchar *name,
		  gpointer user_data)
{
}

static void
on_name_lost (GDBusConnection *connection,
	      const gchar *name,
	      gpointer user_data)
{
  g_main_loop_quit (mainloop);
}

gint 
main (gint argc, gchar **argv)
{
  guint owner_id;

  g_type_init();
  
  introspection_data = g_dbus_node_info_new_for_xml (_webapps_service, NULL);
  g_assert (introspection_data != NULL);
  
  owner_id = g_bus_own_name (G_BUS_TYPE_SESSION, "com.canonical.Unity.Webapps.Service", 
			     G_BUS_NAME_OWNER_FLAGS_NONE,
			     on_bus_acquired,
			     on_name_acquired,
			     on_name_lost,
			     NULL /* userdata */,
			     NULL /* userdata free*/);
			     
  
  mainloop = g_main_loop_new(NULL, FALSE);
  g_main_loop_run(mainloop);
  
  g_bus_unown_name(owner_id);
  g_dbus_node_info_unref (introspection_data);

  return 0;
}
